MAUI_View myWinder; 
MAUI_Button whistler;
MAUI_Button nox[3]; // note on/off, exit
MAUI_Button fms[3]; // fast/medium/slow
MAUI_Slider inGain;
MAUI_Slider sliders[8];
setupGUI();

// Audio low-pass breath noise detector
// adc => FIR lp => OnePole op => blackhole;
// lp => op; 3 => op.op; 0.999 => op.pole; 5.0 => op.gain;
// 512.0 => lp.sinc; // set very low frequency cutoff
adc => LPF lp2 => LPF lp => OnePole op => blackhole; // if you don't have FIR Chugin
lp => op;  3 => op.op; 0.999 => op.pole; 500.0 => op.gain;
40 => lp.freq => lp2.freq;
3 => lp.Q => lp2.Q;

0.0 => float blow;
0.0 => float blowin;

spork ~ updateBreath();

1 => int notDone;

while ( notDone )  {
    0.1 :: second => now;
    inGain.value() => lp.gain;
    if (nox[2].state()) 0 => notDone;
    blow => sliders[0].value;
}

fun void updateBreath()  {
    while (notDone)  {
        0.01::second => now;
        if (op.last() > blow)  {
            op.last()*0.01+blow*0.99 => blow; // slower attack
        }
        else {
            op.last()*0.2+blow*0.8 => blow; // faster release
        }
        if (blow > 1.0) 1.0 => blow;
        blow + sliders[1].value() => blowin;
        if (blowin > 1.0) 1.0 => blowin;
        sendMSG("AfterTouch       0.0 1 "+Std.ftoa(blowin*127,3)+"\n");
    }
}

fun void setupGUI()  {
    myWinder.size(400,370);myWinder.position(0,500);
    myWinder.name("Whistle Blower!!"); myWinder.display();
    whistler.size(150,130); whistler.setImage(me.dir()+"/tcl/bitmaps/prcFunny.png");
    myWinder.addElement(whistler);
    inGain.size(80,150);inGain.orientation(2);inGain.position(120,0);
    inGain.name("gain");inGain.range(0.0,3.0);1.0 => inGain.value;
    myWinder.addElement(inGain);
    for (0 => int i; i < 3; i++) { 
        nox[i].size(100,75);nox[i].position(170+70*i,0); 
        fms[i].size(100,80);fms[i].position(170+70*i,50);
        myWinder.addElement(nox[i]);
        myWinder.addElement(fms[i]);
        spork ~ watchButt(fms[i]);
    }
    for (0 => int i; i < 4; i++) { 
        sliders[i].size(200,80);sliders[i+4].size(200,80);
        sliders[i].range(0.0,1.0); sliders[i+4].range(0.0,1.0);
        sliders[i].position(0,120+60*i);sliders[i+4].position(200,120+60*i);
        myWinder.addElement(sliders[i]);myWinder.addElement(sliders[i+4]);
        spork ~ watchSlider(sliders[i]); spork ~ watchSlider(sliders[i+4]);
    }
    nox[0].name("NoteOn");nox[1].name("NoteOff");nox[2].name("Exit");
    fms[0].name("Normal");fms[1].name("Slow");fms[2].name("Really\nSlow");
    sliders[0].name("Blow (mic) Press."); 0.01 => sliders[0].value;
    sliders[1].name("Pressure Bias");     0.0 => sliders[1].value;
    sliders[2].name("Whistle Freq.");     0.57 => sliders[2].value;
    sliders[3].name("Blow Freq. Mod.");   0.5 => sliders[3].value;
    sliders[4].name("Breath Noise");      0.5 => sliders[4].value;
    sliders[5].name("Fipple Freq. Mod."); 0.5 => sliders[5].value;
    sliders[6].name("Fipple Amp. Mod.");  0.5 => sliders[6].value;
    sliders[7].name("Reverb Mix");        0.1 => sliders[7].value;
    
    sendMSG("AfterTouch       0.0 1 4.000000\n");
    sendMSG("NoteOn        0.0 1 73.800 4.000\n");
    sendMSG("ControlChange    0.0 1  1 64.000000\n");
    sendMSG("ControlChange    0.0 1  2 64.000000\n");
    sendMSG("ControlChange    0.0 1  4 64.000000\n");
    sendMSG("ControlChange    0.0 1  11 64.000000\n");
    sendMSG("ControlChange    0.0  1  44 12.000000\n");
}

fun void watchButt(MAUI_Button aButt)  {
    while (notDone)  {
        aButt => now;
        if (aButt == fms[0]) sendMSG("ControlChange    0.0 1  64 1.0 \n");
        else if (aButt == fms[1]) sendMSG("ControlChange    0.0 1  64 10.0 \n");
        else if (aButt == fms[2]) sendMSG("ControlChange    0.0 1  64 20.0 \n");
    }
}

fun void watchSlider(MAUI_Slider aSlider)  {
    string temp;
    while (notDone)  {
        aSlider => now;
        if (aSlider == sliders[0] || aSlider == sliders[1]) {
            sendMSG("AfterTouch       0.0 1 "+Std.ftoa(blowin*127,3)+"\n");
        }
        else if (aSlider == sliders[1]) sendMSG("HOWDY!!");
        else if (aSlider == sliders[2])  {
            "NoteOn        0.0 1 " => temp;
            temp+Std.ftoa(sliders[2].value()*127.0,3)+" " => temp;
            temp+Std.ftoa(blowin*127.0,3)+"\n" => temp;
            sendMSG(temp);
        }
        else if (aSlider == sliders[3]) ;
        else if (aSlider == sliders[4]) ;
        else if (aSlider == sliders[5]) ;
        else if (aSlider == sliders[6]) ;
        else if (aSlider == sliders[7]) ;
    }
}

fun void sendMSG(string aString)  {
    chout <= aString;
    chout.flush();
}

